#!/usr/bin/env node
"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var arg_1 = __importDefault(require("arg"));
var child_process_1 = require("child_process");
var console_1 = require("console");
var fs_1 = require("fs");
var path_1 = require("path");
var prettier_1 = require("prettier");
var args = arg_1["default"]({ '--update': Boolean, '--tsConfigFileName': String });
var tsConfigFileName = args['--tsConfigFileName'] || 'tsconfig.json';
var packages = JSON.parse(child_process_1.execSync("lerna ls --json --loglevel silent").toString());
var packagesDirectory = path_1.dirname(packages[0].location);
packages.forEach(function (_a) {
    var name = _a.name, location = _a.location;
    child_process_1.exec('npm ls --depth 0 --json --silent', { cwd: location }, function (_error, stdout, _stderror) {
        var dependencyNames = Object.keys(JSON.parse(stdout).dependencies || {});
        var paths = dependencyNames.map(getPathForDependency).filter(function (x) { return !!x; });
        if (paths.length == 0) {
            return console_1.log(name + " - 0 dependencies found.");
        }
        var configFilePath = path_1.join(location, tsConfigFileName);
        var config = JSON.parse(fs_1.readFileSync(configFilePath).toString());
        var _a = config.references, references = _a === void 0 ? [] : _a;
        var existingPaths = references.map(function (r) { return r.path; });
        if (paths.sort().toString() === existingPaths.sort().toString()) {
            console_1.log(name + " - " + paths.length + " dependencies found. All match project references in " + tsConfigFileName + ".");
        }
        else if (args['--update']) {
            console_1.warn("*UPDATE* - " + name + " - Some dependencies didn't match. Writing " + paths.length + " to " + tsConfigFileName + ".");
            var newJsonConfig = JSON.stringify(__assign({}, config, { references: paths.map(function (path) { return ({ path: path }); }) }));
            fs_1.writeFileSync(configFilePath, prettier_1.format(newJsonConfig, { parser: 'json' }));
        }
        else {
            console_1.error("**FAIL** - " + name + " - Some dependencies didn't match.");
            console_1.error("**FAIL** - " + name + " - From npm packages: " + paths);
            console_1.error("**FAIL** - " + name + " - From " + tsConfigFileName + ": " + existingPaths);
            process.exit(1);
        }
    });
});
var doesTsConfigFileExist = function (_a) {
    var location = _a.location;
    return fs_1.existsSync(path_1.join(location, tsConfigFileName));
};
function getRelativeTsConfigFilePath(_a) {
    var location = _a.location;
    var relativePath = location.replace(packagesDirectory, '..');
    var file = tsConfigFileName == 'tsconfig.json' ? '' : tsConfigFileName;
    return path_1.join(relativePath, file);
}
function getPathForDependency(name) {
    var pkg = packages.find(function (p) { return p.name === name; });
    return pkg && doesTsConfigFileExist(pkg) ? getRelativeTsConfigFilePath(pkg) : '';
}
